# package
from .mapper import Document
from .mapper import TextProperty, MultiTextProperty
from .mapper import CompositeTextProperty, MultiCompositeTextProperty
from .mapper import ReferenceProperty, FileProperty, BooleanProperty
from .couchdb import CouchdbDocument